"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MotifSpace = void 0;
const tslib_1 = require("tslib");
const abstract_signer_1 = require("@ethersproject/abstract-signer");
const typechain_1 = require("@motif-foundation/asset/dist/typechain");
const addresses_1 = require("./addresses");
const wallet_1 = require("@ethersproject/wallet");
const providers_1 = require("@ethersproject/providers");
const utils_1 = require("./utils");
class MotifSpace {
    constructor(signerOrProvider, chainId) {
        if (abstract_signer_1.Signer.isSigner(signerOrProvider)) {
            this.readOnly = false;
        }
        else {
            this.readOnly = true;
        }
        this.signerOrProvider = signerOrProvider;
        this.chainId = chainId;
        const network = (0, utils_1.chainIdToNetworkName)(chainId);
        this.spaceAddress = addresses_1.addresses[network].space;
        this.spaceExchangeAddress = addresses_1.addresses[network].spaceExchange;
        this.space = typechain_1.SpaceFactory.connect(this.spaceAddress, signerOrProvider);
        this.spaceExchange = typechain_1.SpaceExchangeFactory.connect(this.spaceExchangeAddress, signerOrProvider);
    }
    /*********************
     * Space Read Methods
     *********************
     */
    fetchContentHash(spaceId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.space.tokenContentHashes(spaceId);
        });
    }
    fetchMetadataHash(spaceId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.space.tokenMetadataHashes(spaceId);
        });
    }
    fetchContentURI(spaceId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.space.tokenURI(spaceId);
        });
    }
    fetchMetadataURI(spaceId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.space.tokenMetadataURI(spaceId);
        });
    }
    fetchIsPublic(spaceId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.space.isPublic(spaceId);
        });
    }
    fetchLands(spaceId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.space.lands(spaceId);
        });
    }
    fetchCreator(spaceId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.space.tokenCreators(spaceId);
        });
    }
    fetchCurrentBidShares(spaceId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.spaceExchange.bidSharesForToken(spaceId);
        });
    }
    fetchCurrentAsk(spaceId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.spaceExchange.currentAskForToken(spaceId);
        });
    }
    fetchCurrentBidForBidder(spaceId, bidder) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.spaceExchange.bidForTokenBidder(spaceId, bidder);
        });
    }
    fetchPermitNonce(address, spaceId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.space.permitNonces(address, spaceId);
        });
    }
    fetchMintWithSigNonce(address) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.space.mintWithSigNonces(address);
        });
    }
    // /*********************
    //  *  Space Write Methods
    //  *********************
    //  */
    // public async updateContentURI(
    //   spaceId: BigNumberish,
    //   tokenURI: string
    // ): Promise<ContractTransaction> {
    //   try {
    //     this.ensureNotReadOnly();
    //     validateURI(tokenURI);
    //   } catch (err) {
    //     return Promise.reject(err.message);
    //   }
    //   return this.space.updateTokenURI(spaceId, tokenURI);
    // }
    // public async updateMetadataURI(
    //   spaceId: BigNumberish,
    //   metadataURI: string
    // ): Promise<ContractTransaction> {
    //   try {
    //     this.ensureNotReadOnly();
    //     validateURI(metadataURI);
    //   } catch (err) {
    //     return Promise.reject(err.message);
    //   }
    //   return this.space.updateTokenMetadataURI(spaceId, metadataURI);
    // }
    // public async mint(
    //   spaceData: SpaceData,
    //   bidShares: BidShares
    // ): Promise<ContractTransaction> {
    //   try {
    //     this.ensureNotReadOnly();
    //     validateURI(spaceData.metadataURI);
    //     validateURI(spaceData.tokenURI);
    //     validateBidShares(
    //       bidShares.creator,
    //       bidShares.owner,
    //       bidShares.prevOwner
    //     );
    //   } catch (err) {
    //     return Promise.reject(err.message);
    //   }
    //   const gasEstimate = await this.space.estimateGas.mint(spaceData, bidShares);
    //   const paddedEstimate = gasEstimate.mul(110).div(100);
    //   return this.space.mint(spaceData, bidShares, {
    //     gasLimit: paddedEstimate.toString(),
    //   });
    // }
    // public async setAsk(
    //   spaceId: BigNumberish,
    //   ask: Ask
    // ): Promise<ContractTransaction> {
    //   try {
    //     this.ensureNotReadOnly();
    //   } catch (err) {
    //     return Promise.reject(err.message);
    //   }
    //   return this.space.setAsk(spaceId, ask);
    // }
    // public async setBid(
    //   spaceId: BigNumberish,
    //   bid: Bid
    // ): Promise<ContractTransaction> {
    //   try {
    //     this.ensureNotReadOnly();
    //   } catch (err) {
    //     return Promise.reject(err.message);
    //   }
    //   return this.space.setBid(spaceId, bid);
    // }
    // public async removeAsk(spaceId: BigNumberish): Promise<ContractTransaction> {
    //   try {
    //     this.ensureNotReadOnly();
    //   } catch (err) {
    //     return Promise.reject(err.message);
    //   }
    //   return this.space.removeAsk(spaceId);
    // }
    // public async removeBid(spaceId: BigNumberish): Promise<ContractTransaction> {
    //   try {
    //     this.ensureNotReadOnly();
    //   } catch (err) {
    //     return Promise.reject(err.message);
    //   }
    //   return this.space.removeBid(spaceId);
    // }
    // public async acceptBid(
    //   spaceId: BigNumberish,
    //   bid: Bid
    // ): Promise<ContractTransaction> {
    //   try {
    //     this.ensureNotReadOnly();
    //   } catch (err) {
    //     return Promise.reject(err.message);
    //   }
    //   return this.space.acceptBid(spaceId, bid);
    // }
    // public async revokeApproval(
    //   spaceId: BigNumberish
    // ): Promise<ContractTransaction> {
    //   try {
    //     this.ensureNotReadOnly();
    //   } catch (err) {
    //     return Promise.reject(err.message);
    //   }
    //   return this.space.revokeApproval(spaceId);
    // }
    // public async burn(spaceId: BigNumberish): Promise<ContractTransaction> {
    //   try {
    //     this.ensureNotReadOnly();
    //   } catch (err) {
    //     return Promise.reject(err.message);
    //   }
    //   return this.space.burn(spaceId);
    // }
    // **********************
    //  * ERC-721 Read Methods
    //  ***********************
    fetchBalanceOf(owner) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.space.balanceOf(owner);
        });
    }
    fetchOwnerOf(spaceId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.space.ownerOf(spaceId);
        });
    }
    fetchSpaceOfOwnerByIndex(owner, index) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.space.tokenOfOwnerByIndex(owner, index);
        });
    }
    fetchTotalSpace() {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.space.totalSupply();
        });
    }
    fetchSpaceByIndex(index) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.space.tokenByIndex(index);
        });
    }
    fetchApproved(spaceId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.space.getApproved(spaceId);
        });
    }
    fetchIsApprovedForAll(owner, operator) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.space.isApprovedForAll(owner, operator);
        });
    }
    /***********************
     * ERC-721 Write Methods
     ***********************
     */
    approve(to, spaceId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.space.approve(to, spaceId);
        });
    }
    setApprovalForAll(operator, approved) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.space.setApprovalForAll(operator, approved);
        });
    }
    setApprovalForAllCustom(operator, approved, spaceAddress) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            const path = `${process.cwd()}/.env.prod`;
            yield require("dotenv").config({ path });
            const provider = new providers_1.JsonRpcProvider(process.env.RPC_ENDPOINT);
            const wallet = new wallet_1.Wallet(`0x${process.env.PRIVATE_KEY}`, provider);
            const space_ = typechain_1.SpaceFactory.connect(spaceAddress, wallet);
            return space_.setApprovalForAll(operator, approved);
        });
    }
    transferFrom(from, to, spaceId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.space.transferFrom(from, to, spaceId);
        });
    }
    safeTransferFrom(from, to, spaceId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.space.safeTransferFrom(from, to, spaceId);
        });
    }
    /****************
     * Miscellaneous
     * **************
     */
    // public eip712Domain(): EIP712Domain {
    //   // Due to a bug in ganache-core, set the chainId to 1 if its a local blockchain
    //   // https://github.com/trufflesuite/ganache-core/issues/515
    //   const chainId = this.chainId == 50 ? 1 : this.chainId;
    //   return {
    //     name: "Motif",
    //     version: "1",
    //     chainId: chainId,
    //     verifyingContract: this.spaceAddress,
    //   };
    // }
    // public async isValidBid(spaceId: BigNumberish, bid: Bid): Promise<boolean> {
    //   const isAmountValid = await this.spaceExchange.isValidBid(
    //     spaceId,
    //     bid.amount
    //   );
    //   const decimal100 = Decimal.new(100);
    //   const currentBidShares = await this.fetchCurrentBidShares(spaceId);
    //   const isSellOnShareValid = bid.sellOnShare.value.lte(
    //     decimal100.value.sub(currentBidShares.creator.value)
    //   );
    //   return isAmountValid && isSellOnShareValid;
    // }
    // public isValidAsk(spaceId: BigNumberish, ask: Ask): Promise<boolean> {
    //   return this.spaceExchange.isValidBid(spaceId, ask.amount);
    // }
    // public async isVerifiedSpace(
    //   spaceId: BigNumberish,
    //   timeout: number = 10
    // ): Promise<boolean> {
    //   try {
    //     const [tokenURI, metadataURI, contentHash, metadataHash, isPublic, lands] =
    //       await Promise.all([
    //         this.fetchContentURI(spaceId),
    //         this.fetchMetadataURI(spaceId),
    //         this.fetchContentHash(spaceId),
    //         this.fetchMetadataHash(spaceId),
    //         this.fetchIsPublic(spaceId),
    //         this.fetchLands(spaceId)
    //       ]);
    //     const spaceData = constructSpaceData(
    //       tokenURI,
    //       metadataURI,
    //       contentHash,
    //       metadataHash,
    //       isPublic,
    //       lands
    //     );
    //     return isSpaceDataVerified(spaceData, timeout);
    //   } catch (err) {
    //     return Promise.reject(err.message);
    //   }
    // }
    /******************
     * Private Methods
     ******************
     */
    ensureNotReadOnly() {
        if (this.readOnly) {
            throw new Error("ensureNotReadOnly: readOnly Motif instance cannot call contract methods that require a signer.");
        }
    }
}
exports.MotifSpace = MotifSpace;
//# sourceMappingURL=MotifSpace.js.map